/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Copyright 2006 Simon Newton
 *  nomis52@westnet.com.au
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <sandnet/sandnet.h>
#include <sandnet/packets.h>

#define DEFAULT_DEVICE "/dev/dmx0"
#define DEFAULT_DEVICE2 "/dev/dmx1"
#define DEFAULT_DEVICE3 "/dev/dmx2"
#define DEFAULT_DEVICE4 "/dev/dmx3"

#define UNCONNECTED -1
#define SHORT_NAME "NWAN"
#define LONG_NAME "Netgear Wireless Artnet Node"


typedef struct {
  int verbose;
  char *ip_addr;
  int subnet_addr;
} opts_t;



/*
 * Called when we have dmx data pending
 */
int dmx_handler(sandnet_node n, uint8_t group, uint8_t universe, int length, uint8_t *data,  void *d) {


  printf("got dmx %i %i %i 0x%hhx 0x%hhx 0x%hhx 0x%hhx 0x%hhx \n",length, group, universe ,  data[0],   data[1],   data[2],   data[3],   data[4]  );

  return 0;
}



/*
 * Main function
 *
 */
int run(opts_t *ops) {
  sandnet_node node;

  // create new sandnet node, and set config values
    node = sandnet_new(ops->ip_addr, ops->verbose);;

  sandnet_set_dmx_handler(node, dmx_handler, (void*) ops);

  if ( sandnet_start(node) ) {
    printf("error %s\n" , sandnet_strerror() );
    exit(1);
  }

  while(1) {
    // set a 2 second timeout on the read
    // this way we send a ad frame every second
    // even if we don't get any sandnet packets
    sandnet_read(node, 2);
    sandnet_send_ad(node);

  }

  return 0;
}


/*
 * Set our default options, command line args will overide this
 */
void init_ops(opts_t *ops) {

  ops->verbose = 0;
  ops->ip_addr = NULL;
  ops->subnet_addr = 0;

}

/*
 * Parse command lines options and save in opts_s struct
 *
 */
void parse_args(opts_t *ops, int argc, char *argv[]) {
  // parse args
  int optc;

  // parse options
  while ((optc = getopt (argc, argv, "a:v")) != EOF) {
    switch (optc) {
       case 'a':
        free(ops->ip_addr);
        ops->ip_addr = (char *) strdup(optarg);
            break;
      case 'v':
        ops->verbose = 1;
            break;
          default:
        break;
      }
  }
}


int main(int argc, char *argv[]) {
  opts_t ops;

  init_ops(&ops);
  parse_args(&ops, argc, argv);

  run(&ops);

  return 0;
}

